#!/bin/sh
# Copyright (c) 2002, Intel Corporation. All rights reserved.
# This file is licensed under the GPL license.  For the full content
# of this license, see the COPYING file at the top level of this
# source tree.

#  Test pthread_rwlockattr_getpshared() fails and an error number is returned if:
#
#  Upon success, it returns 0, and stores the value of the process-shared attribute of 'attr'
#  into 'pshared'.
#

# This is tested implicitly via assertion 1.

echo "Tested implicitly via assertion 1.  See output for status"
exit 0
